//package Aufgabe;

import java.util.List;
import java.lang.reflect.Array;
import java.util.Random;

public abstract class Aufgabe {

	int[][] aktionen;
	int bereichx;
	int ssthresh;
	int cwndergebnis;
	final int Timeout=0;
	final int TDACK=1;

	abstract void send(List SendParameter);

	abstract void save();

	abstract boolean doCheck(List controls);


	/** Erzeugung der zufälligen Zeitpunkte und Ereignisse in diesen
	  * Zeitpunkten. Abhängig von den Benutzereingaben für Anzahl der Ereignisse
	  * und den betrachteten Gesamtzeitraum werden die erzeugten Daten in ein
	  * zweisimensionales, globales Array geladen.*/
	public Aufgabe(int bereichx, int anzahl)
	{
		this.bereichx = bereichx;
		ssthresh=64;
		// Zufallsgenerator zur Erzeugung der Aktionen und der
		// Aktionszeitpunkte.
		Random generator = new Random();
		// Array zum Zwischenspeichern der Aktionen und Aktionszeitpunkte
		aktionen = new int[anzahl][2];

		// Festlegen der Aktionen und der Aktionszeitpunkte.
		for (int i=0; i<anzahl; i++)
		{
			// Zeitpunkt
			if (i==0)
				aktionen[i][0] = 1 + generator.nextInt((bereichx)/anzahl);
			else
				aktionen[i][0] = 1 + aktionen[i-1][0] + generator.nextInt(bereichx-aktionen[i-1][0]-anzahl+i+1);
			// Aktion. Mögliche Aktionen: TDACK oder Timeout
			// Timeout = 0, TDACK = 1;
			aktionen[i][1] = generator.nextInt(2);
		}
	}


	/** Berechnung des Congestion Window bei gegebener alter cwnd
	  * und den Angaben, ob ein Timeout stattfand oder ein Paketverlust
	  * mittels TDACK entdeckt wurde. */
	protected int getCongestionWindow(int cwnd, boolean vTDACK, boolean vTimeout)
	{
		int rueckgabe=1;
		// Berechnung der Congestion Window Größe cwnd
		// Übergabeparameter: alte cwnd, timeout ja/nein, TDACK ja/nein, ssthresh
		if ((cwnd < ssthresh) && (!vTimeout) && (!vTDACK))
		  // Algorithmus in Slow Start Phase
		  // neues cwnd=cwnd*2
		  rueckgabe= cwnd*2;
		if (vTimeout && (!vTDACK))
		{
		  // Algorithmus in Slow Start Phase oder Congestion Avoidence
		  // neues cwnd=1, da timeout
		  // neues ssthresh=cwnd/2, da Paketverlust
		  ssthresh = cwnd/2;
		  rueckgabe= 1;
		}
		if (vTDACK && (!vTimeout))
		{
		  // Algorithmus in Slow Start Phase oder Congestion Avoidence
		  // neues cwnd = cwnd/2, da Paketverlust mit TDACK
		  // neues ssthresh = cwnd/2, da Paketverlust
		  ssthresh=cwnd/2;
		  rueckgabe=cwnd/2;
		}
		if ((cwnd >= ssthresh) && (!vTimeout) && (!vTDACK))
		  // Übergang zu Congestion Avoidence
		  // da kein Paketverlust: cwnd = cwnd +1
		  rueckgabe=cwnd +1;

		// minimales CongestionWindow ist 1.
		if (rueckgabe == 0)
			rueckgabe = 1;

		// minimales ssthresh
		if (ssthresh == 0)
			ssthresh=1;

		return rueckgabe;
	}


	/** Berechnung des gesamten Congestion Window nach einem festgelegten,
	  * übergebenen Zeipunkt durch wiederholtes Aufrufen der Methode
	  * getCongestionWindow */
	protected int getCongestionWindowResult(int runde)
	{
		int cwndneu=1;
		int cwndalt=1;
		int j;
		ssthresh=64;
		for (int i = 1 ; i <= runde ; i++)
		{
			// wenn x enthalten in Array Aktionen, hat eine Aktion
			// stattgefunden -> Timeout oder TDACK
			j=inArray(aktionen, i);
			if (j >= 0)
			{
				// Eintrag in aktionen gibt Stelle der Art der Aktion
				// in Arten an.
				if (aktionen[j][1] == Timeout)
				  // wenn Aktion = Timeout -> vTimeout true, vTDACK false
				  cwndneu = (int)getCongestionWindow(cwndalt,false,true);
				else
				  // sonst Aktion = TDACK -> vTDACK true, vTimeout false
				  cwndneu = (int)getCongestionWindow(cwndalt,true,false);
			 }
			 else
				// kein Aktionszeitpunkt, also nichts passiert
				// damit vTimeout false, vTDACK false
				cwndneu = (int)getCongestionWindow(cwndalt, false,false);
			cwndalt=cwndneu;
		}
		cwndergebnis=cwndalt;
		return cwndalt;
	}



	protected int inArray (int[][] array, int x)
	{
		int ergebnis=-1;
		 for (int i=0; i<Array.getLength(array); i++)
		 {
		   if (array[i][0] == x)
			ergebnis = i;
		 }
		  return ergebnis;
	}

}